#!/usr/bin/python

version_number = "1.9.2"
release_date = "Wed Sep 30 2003"
#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2003
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit www.gnu.org
#


from .ParamParser import *
from .Executor import *
from .MultiAgrizer import *
from .Agrizer import *
from .TeXParser import *

from .MeanCalculation import *
from .MatrixPlotter import *



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import

#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2003
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit www.gnu.org
#

# BROKEN
# BROKEN
# BROKEN
# BROKEN
# BROKEN
#
#

from builtins import next
from builtins import str
from builtins import range
import sys
import os.path
import os

from .ParamParser import *
from . import Load
from math import *
from . import PyGrace


class Agrizer(ParamParser):
    box_size = 1.
    min_value = None
    xcol = 0
    ycol = 1
    xformula = "x"
    yformula = "y"
    xlabel = "x"
    ylabel = "y"
    xscale = "Normal"
    yscale = "Normal"
    minx = 0.
    maxx = 1.
    miny = 0.
    maxy = 1.

    title = ""

    def agr(self, outname, inname):

        title = self.title
        elpath = ""
        ac_values = self.actual_values()
        for actual in ac_values:
            title += " %s=%s " % actual
            legend = " %s = %s " % actual

            if actual[0] in self.isvariable[:-1]:
                elpath += "%s-%s/" % actual

        try:
            if not os.path.exists(elpath):
                raise AssertError("")
        except:
            print("Error accesing path: '%s'." % elpath)
            sys.exit()
        cwd = os.getcwd()
        os.chdir(elpath)

        sys.stdout = open(outname, "w")

        ls = Load.loadXY(inname, self.xcol, self.ycol)
        ls2 = Load.transformXY(ls, self.xformula, self.yformula)

        g1 = PyGrace.GraceDocument()
        g1.set_data(ls2, legend)

        g1.autoscale()
        g1.set_labels(self.xlabel, self.ylabel)
        g1.set_scale(self.xscale, self.yscale)
        g1.set_title(title)
        g1.dump()

        os.chdir(cwd)

    def agr(self, pattern, outname=None, autoscale="xy"):

        title = self.title
        elpath = ""
        ac_values = self.actual_values()
        for actual in ac_values:

            if actual[0] in self.isvariable[:-1]:
                title += " %s=%s " % actual
                legend = " %s = %s " % actual
                elpath += "%s-%s/" % actual

        try:
            if not os.path.exists(elpath):
                raise AssertError("")
        except:
            print("Error accesing path: '%s'." % elpath)
            sys.exit()
        cwd = os.getcwd()
        os.chdir(elpath)
        if type(pattern) is type(""):
            pattern = [pattern]
        g1 = PyGrace.GraceDocument()
        for inname in pattern:
            for i in self.var:
                inname = inname.replace(
                    "{%s}" % i, "%s-%s" % (i, str(self.act_val(i))))

            if os.path.isfile(inname):
                if not outname:
                    outname = inname + ".agr"

                else:
                    for i in self.var:
                        outname = outname.replace(
                            "{%s}" % i, "%s-%s" % (i, str(self.act_val(i))))

                ls = Load.loadXY(inname, self.xcol, self.ycol)
                ls2 = Load.transformXY(ls, self.xformula, self.yformula)
                g1.set_data(ls2, legend)
                g1.set_labels(self.xlabel, self.ylabel)
                g1.set_scale(self.xscale, self.yscale)
                g1.set_title(title + inname)
                g1.set_world(self.minx, self.maxx, self.miny, self.maxy)
                g1.autoscale(autoscale)
                g1.autoscale()
                sys.stdout = open(outname, "w")
                g1.dump()

            else:
                sys.stderr.write(
                    "skipping " + inname + "... file does not exist\n")

        os.chdir(cwd)

    def hist(self, pattern, mustCalculateDif, outname=None, autoscale="xy"):

        title = self.title
        elpath = ""
        ac_values = self.actual_values()

        for actual in ac_values:

            if actual[0] in self.isvariable[:-1]:
                elpath += "%s-%s/" % actual
                title += " %s = %s " % actual

        legend = title
        cwd = os.getcwd()
        try:
            if not os.path.exists(elpath):
                raise AssertError("")
        except:
            print("Error accesing path: '%s'." % elpath)
            sys.exit()

        os.chdir(elpath)
        if type(pattern) is type(""):
            pattern = [pattern]
        g1 = PyGrace.GraceDocument()
        isSomething = 0  # false the document is void
        if not outname:
            outname = pattern[0]
        for i in self.var:
            outname = outname.replace(
                "{%s}" % i, "%s-%s" % (i, str(self.act_val(i))))

        for inname in pattern:
            for i in self.var:
                inname = inname.replace(
                    "{%s}" % i, "%s-%s" % (i, str(self.act_val(i))))

#      sys.stderr.write(os.path.abspath(inname)+"... %d \n"%os.path.isfile(inname))
            if os.path.isfile(inname):
                isSomething = 1  # true the document is NOT void

                ls = Load.loadY(inname, 0)
                if mustCalculateDif:

                    ls = [
                        ls[i + 1] - ls[i]
                        for i in range(len(ls) - 1)
                    ]
                from . import histogram

                hist = histogram.SPGHistogram(self.box_size)

                for x in ls:
                    hist.add_value(x)

                ls2 = hist.get_dataset()

                g1.set_data(ls2, legend, "bar")
            else:
                sys.stderr.write(
                    "skipping " + inname + "... file does not exist\n")
        if isSomething:
            sys.stdout = open(outname, "w")
            g1.set_labels(self.xlabel, self.ylabel)
            g1.set_scale(self.xscale, self.yscale)
            g1.set_title(title)
            g1.set_world(self.minx, self.maxx, self.miny, self.maxy)
            g1.autoscale(autoscale)
            g1.dump()
        os.chdir(cwd)

    def doit(self, outname="out.agr", inname="out.dat", pattern=None, patternname=None, autoscale="xy"):
        flag = 1

        while flag:
            if not pattern:
                self.agr(outname, inname)
            else:
                self.agr(pattern, patternname)

            flag = next(self)
        self.reset()

    def dohist(self, pattern, mustCalculateDif=0, patternname=None, autoscale="xy"):
        flag = 1

        while flag:
            self.hist(pattern, mustCalculateDif, patternname)
            flag = next(self)
        self.reset()



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import

#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2005
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit www.gnu.org
#

from builtins import str
from builtins import object
import sys
import os.path
import os

from .ParamParser import *
from math import *


class Executor(object):
    #
    # :::~ This variable specifies how many VARIABLEs will be kept off
    #      the directory heirarchy (NOTE THAT MUST HAVE A MINUS SIGN)
    #

    def __init__(self, lsLines):
        self.parser = ParamParser(lsLines)
        self.limit = -1

    #
    # :::~ This variable specifies that your prigram will understand correctly
    #      the print command inside the input of your program
    #
    def run(self, exename, outname="out.dat", inname="in.dat"):
        """
        runs the program 'exename', that receives its input from
        'inname'. Outputs in file 'outname'. This is done for the current
        value of parameters
        """

        cwd = os.path.abspath(os.path.curdir)

        way2 = self.parser.directory_tree(self.limit)

        try:
            if not os.path.isdir(way2):
                os.makedirs(way2)
        except:
            print("Error!! creating directory: '%s'" % way2)
            sys.exit(1)

        os.chdir(way2)

        fparam = open(inname, "w")
        fparam.write(str(self.parser))
        fparam.close()

        open(outname, "a").write(self.parser.output_tree(self.limit) + "\t")

        os.system("%s >> %s " % (exename, outname))

        os.chdir(cwd)

    def doit(self, command, inname="in.dat", outfile="out.dat"):
        """
          runs systematically command (which is a OS command or executable). In the correspondig directory
          inname is the input file for the command
          while outfile is the output file name
        """
        for i in self.parser:
            self.run(command, outfile, inname)


#
#
#
#



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

from builtins import range
from builtins import object
from past.utils import old_div
import math

class SPGHistogram(object):
    __boxsize = 1
    __dict = {}
    __elements = 0

    def __init__(self, boxsize=1):
        self.__boxsize = boxsize
        self.__elements = 0.
#    import sys
#    sys.stderr.write("histogram box size = %lf\n"%self.__boxsize)

    def __getitem__(self, value):

        nearest = self.__get_nearest(value)

        if nearest in self.__dict:
            return old_div(self.__dict[nearest], self.__elements)
        else:
            return 0.

    def add_value(self, value):
        nearest = self.__get_nearest(value)
        if nearest in self.__dict:
            self.__dict[nearest] += 1.
        else:
            self.__dict[nearest] = 1.
        self.__elements += 1.

    def __str__(self):
        if self.__elements == 0:
            return " "
        keys = list(self.__dict.keys())
        keys.sort()
        return "\n".join([
                         "%f\t%f" % (k, (old_div(self.__dict[k], self.__elements)))
                         for k in keys
                         ])

    def __get_nearest(self, value):
        return self.__boxsize * math.floor(old_div(value, self.__boxsize))

    def get_dataset(self):
        if self.__elements == 0:
            return [[0, 0]]
        keys = list(self.__dict.keys())
        keys.sort()
        return [
            [k, (old_div(self.__dict[k], self.__elements))]
            for k in keys
        ]


if __name__ == "__main__":
    a = SPGHistogram(.1)
    import random
    import sys
    import time

    time1 = time.time()
    for i in range(100000):
        a.add_value(random.gauss(0.1, 0.5))
    time2 = time.time()

    print(a)
    sys.stderr.write("ellapsed time: %lf\n" % (time2 - time1))



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import

version_number = "1.9.9"
release_date = "Thu Jan 07 01:24:18 CEST 2005"
#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2005
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit http://www.gnu.org
#

from builtins import object
from builtins import next
from . import ParamIterators

#:::~ Important: for constants and functions already defined
from math import *

#
#
#
#


class ParamParser(object):

    """
      Initialized with a list of strings, each one containing commands.
      Each line will have a syntax as follows
      (iterator_type) (variable_name) [bounds]
      (iterator_type) can be one of the following characters
      '+' '-' '*' '/'  => all of them expect bounds given by [min_value] [max_value] [step]
      '.'              => punctual iterator, (bounds) is in fact a (BLANK separated) list with all the possible values
      ':'              => defines a CONSTANT (i.e. not iterable object)
      '#'              => repetition operator

    """

#:::~ This variable specifies the list separator used while parsing "param.dat"
    separator = " "

#:::~ A dictionary with all the possible iterators
    iterator_types_dict = {
        "+": ParamIterators.ItOperatorPlus,
        "-": ParamIterators.ItOperatorMinus,
        "*": ParamIterators.ItOperatorProduct,
        "/": ParamIterators.ItOperatorDivision,
        "**": ParamIterators.ItOperatorPower,
        ":": ParamIterators.ItConstant,
        ".": ParamIterators.ItPunctual,
        "#": ParamIterators.ItRepetition
    }

#:::~ a list with an alternative order if your binary does not read a free-style input file
#  alternative_order = []
#:::~ a list with all the varying entities (i.e. those not CONSTANT)
# isvariable = []
# 1
    def __init__(self, lsLines):
        """
          lsLines is a list of commands understood by this class.
        """

        self.entities = []

        self.iterator_list = []

        self.variables_list = []

        self.actual_values = {}

        self.__parse(lsLines)
        import copy
        self.reversed = copy.copy(self.iterator_list)
        self.reversed.reverse()
        self.reset()

# 1
    def __get_iteration_and_command(self, cadena):
        """
          returns the iteration type of a command. The iteration type is defined as the set of non alphanumeric characters
          at the beginning of the line
        """
        last_char = 0
        while not cadena[last_char].isalnum():
            last_char += 1
        return cadena[:last_char], cadena[last_char:]

# 1
    def __parse(self, ls):
        """
          internal function that parses the input
        """
        for sit in ls:
            # strips trailing and leading blanks

            # iteration type

            it_type, str_rest = self.__get_iteration_and_command(sit.strip())

            new_iterator = self.iterator_types_dict[it_type]()

            new_iterator.set_command(str_rest, self.separator)

            self.iterator_list.append(new_iterator)

        self.variables_list = [
            i_iter for i_iter in self.iterator_list if i_iter.is_variable()]
        self.entities = [i_iter.get_varname()
                         for i_iter in self.iterator_list]

    def __next__(self):
        """
         next() iterates over the possible values raising a StopIteration when the possible values
          are exhausted
        """
        if self.is_reset:
            self.is_reset = False
            return self.actual_values

        for i_iter in self.reversed:
            last_iterated = i_iter
            varname = i_iter.get_varname()
            try:
                self.actual_values[varname] = next(i_iter)

                break
            except StopIteration:
                self.actual_values[varname] = i_iter.reset()

                if last_iterated == self.iterator_list[0]:
                    raise StopIteration

        return self.actual_values

# 1
    def reset(self):
        """
          This function resets the iterator to its starting point
        """
        for i_iter in self.iterator_list:
            i_iter.reset()
            self.actual_values[i_iter.get_varname()] = i_iter.get_value()
        self.is_reset = True

    def __iter__(self):
        return self

# 1
    def __str__(self):
        """
          defines how the actual value of the parameter set is printed out.
          A good candidate to be overwritten in inheriting classes.
        """
        thisstr = "\n".join([
                            "%s%s%s" % (
                                k, self.separator, self.actual_values[k])
                            for k in self.entities if k
                            ]) + "\n"
        #:::~ replaces structures of the kind {var} by var-value, very useful for
        #     generation of multiple output files.
        for i_iter in self.iterator_list:
            varname = i_iter.get_varname()
            thisstr = thisstr.replace(
                "{%s}" % varname,
                "%s-%s" % (varname, self.actual_values[varname])
            )

        return thisstr
# 1

    def value_of(self, varn):
        """
          returns the actual value of the variable 'varn'
        """
        try:
            return self.actual_values[varn]
        except ValueError:
            import sys
            sys.stderr.write("'%s' not found among entities\n" % varn)
            sys.exit()

# 1
    def set_order(self, new_order):
        """
          sets a new order for the output.
          May be a subset of the variables, but it can not be a superset
          useful if your program only reads a fixed input file
        """
        try:
            for i in new_order:
                [k.get_varname() for k in self.iterator_list].index(i)
        except ValueError:
            import sys
            sys.stderr.write("error! %s not found among entities\n" % i)
            sys.stderr.write("entities = %s\n" % self.entities)
            sys.exit()
        self.entities = new_order

# 1
    def directory_tree(self, limit=-1):
        """
        returns the directory path conducting to the actual values of the
        parameter set.  by default (limit=-1) the directory tree is extended to
        all the variables list except for the last variable.

        By setting limit to something else, you change the amount of variables
        kept left from the directory generation. (i.e. limit=-2, will leave out
        of the directory path the last two variables)

        """

        import os.path
        thepath = os.path.curdir + os.path.sep

        for i_iter in self.variables_list[:limit]:
            thepath += "%s-%s%s" % (
                i_iter.get_varname(),
                self.actual_values[i_iter.get_varname()],
                os.path.sep)

        return thepath

# 1
    def output_tree(self, limit=-1):
        """
          returns the output from limit given the actual values of the parameter set.
          by default (limit=-1) the output will only print the last variable value.
          By setting limit to something else, you change the amount of variables printed
          (i.e. limit=-2, will print the value of the last two variables)
        """
        theoutput = ""
        for i_iter in self.variables_list[limit:]:
            theoutput += "%s" % self.actual_values[
                i_iter.get_varname()] + self.separator
        return theoutput


#
#
if __name__ == "__main__":
# command=[":a 0","*c 2 6 3",":filename {c}.dat","+d 5 9 4","#2",":end"]
#  command=["*c 2 18 3"]
    command = [":a 0", "*c 2 6 3", ":filename {c}.dat", "+d 5 9 4", ":end"]
#
    print(command)
    pp = ParamParser(command)
   # pp.set_order(["c","a","d"])
    for i in pp:
        print(pp)



from __future__ import absolute_import
#
# :::~ Copyright (C) 2005 by Claudio J. Tessone <tessonec@imedea.uib.es>
#
# Created on: 09 Jan 2005
#
# Copyright: Distributed according to GNU/GPL Version 2
#            (see http://www.gnu.org)
#
#
from builtins import map
from builtins import range
from builtins import object
import sys
from math import *


class SPGIterator(object):
    """This is a subsidiary abstract class for the ParamParser one.  It defines
    the iteration type in the constructor. Must be subclassed (it's abstract)
    In principle there is no need to touch anything here (well, you can always
    improve the code) only you must subclass this object if you wish to add
    other iterator types.
    """

    def __init__(self):
        self.index = 0
        self.varname = False
        self.data = []

    def __iter__(self):
        return self.data.__iter__()

    def is_variable(self):
        return True

    def set_command(self):
        """This is the function called to generate all the possible values of
        the iterator it returns the variable name and all the possible values
        it can take command holds the input line in the parameters file,
        separator is the character delimiting fields (if separator = None, any
        space is considered a separator)
        """
        pass

    def __next__(self):
        self.index += 1
        if self.index == len(self.data):
            raise StopIteration

        return self.data[self.index]

    def reset(self):
        self.index = 0
        return self.data[0]

    def get_varname(self):
        return self.varname

    def get_value(self):
        return self.data[self.index]


class ItOperator(SPGIterator):
    """This subclass generates the values for classes defined according to the
    rule @var_name val_min val_max step where @ is the operation defined for
    the data type.  var_name is the variable name val_min, val_max the bounds
    step  the step the actualization process runs according to actual_value
    = actual_value @ step
    """

    def __init__(self, it_type):
        SPGIterator.__init__(self)
        self.it_type = it_type

    def set_command(self, command, separator=" "):
        str_rest = command.split(separator)
        self.varname = str_rest[0].strip()
        try:
            [xmin, xmax, xstep] = list(map(eval, str_rest[1:]))
        except:
            raise ValueError("Line: '{0}' incorrect number of parameters (found {1}) for iterator       '{2}' over '{3}'".format(
                command, len(str_rest) - 1, self.it_type, self.varname))
            #
            #   Block that raises exception in the case that iteration requested
            #   do not reaches xmax

        try:
            if (xmin < xmax) and (xmin >= eval("%s %s %s" % (xmin, self.it_type, xstep))):
                raise AssertError("")

            if (xmin > xmax) and (xmin <= eval("%s %s %s" % (xmin, self.it_type, xstep))):
                raise AssertError("")
        except:
            raise ValueError("Line: '{0}' Variable '{1}': Error! {2}{3}{4} do not seem to tend to {5}".format(
                command, self.varname, xmin, self.it_type, xstep, xmax))
        #
        #

        lsTmp = []
        xact = xmin

        while((xmin > xmax) ^ (xact <= xmax)):  # ^ is xor in python !
            lsTmp.append(xact)
            xact = eval("%s%s%s" % (xact, self.it_type, xstep))

        self.data = lsTmp


class ItOperatorPlus(ItOperator):

    def __init__(self):
        ItOperator.__init__(self, "+")


class ItOperatorMinus(ItOperator):

    def __init__(self):
        ItOperator.__init__(self, "-")


class ItOperatorProduct(ItOperator):

    def __init__(self):
        ItOperator.__init__(self, "*")


class ItOperatorDivision(ItOperator):

    def __init__(self):
        ItOperator.__init__(self, "/")


class ItOperatorPower(ItOperator):

    def __init__(self):
        ItOperator.__init__(self, "**")


class ItConstant(SPGIterator):
    """This subclass generates a constant "iteration" type.
    """

    def __init__(self):
        SPGIterator.__init__(self)

    def is_variable(self):
        return False

    def set_command(self, command, separator=" "):
        self.varname = command.strip().split(separator)[0]
        self.data = [separator.join(command.strip().split(separator)[1:])]
        self.index = 0


class ItPunctual(SPGIterator):
    """This subclass generates a list of defined values in command command
    should be .var_name value1 value2 value3 ... and so on
    """

    def __init__(self):
        SPGIterator.__init__(self)

    def set_command(self, command, separator=" "):
        str_rest = command.split(separator)
        self.varname = str_rest[0].strip()
#	 self.data = [eval (i.strip()) for i in str_rest[1:] ]
        self.data = [i.strip() for i in str_rest[1:]]


class ItRepetition(SPGIterator):
    """ This subclass generates a list of null with length defined useful when
    trying to repit the run of the program with the same parameters.
    """

    def __init__(self): SPGIterator.__init__(self)

    def set_command(self, command, separator=" "):
        try:
            self.varname = False
            self.data = list(range(eval(command)))
        except:
            raise ValueError(
                "Line: '{0}' iterator '#' could not eval()".format(command))
            sys.exit()

    def is_variable(self):
        return False



#!/usr/bin/env python

from __future__ import print_function
from __future__ import absolute_import

# pyslice.py
#    Copyright (C) 2001  Tim Cera tim@cerazone.net
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name pyslice nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# ===imports======================
import sys
import os
import os.path
import getopt
import time
import subprocess
import re
import shlex
import shutil
import filecmp
import math
# To support montecarlo...
import random

try:
    import threading as _threading
except ImportError:
    import dummy_threading as _threading

import configparser as configparser

from future import standard_library
standard_library.install_aliases()
from builtins import chr
from builtins import map
from builtins import input
from builtins import next
from builtins import str
from builtins import range
from builtins import object

from pyslice.pyslice_lib import PySPG as pyspg
"""
NAME:
    pyslice.py

SYNOPSIS:
    pyslice.py [options]

DESCRIPTION:
    Pyslice creates input data sets from information in pyslice.ini
    and files in the template directory.  Pyslice then runs a command
    within each directory, controlling the processes so that at any
    one time there are less than a certain number of processes.

OPTIONS:
    -h,--help        this message
    -v,--version     version
    -d,--debug       turn on debug messages

EXAMPLES:
    1. As standalone
        pyslice.py
    2. As library
        import pyslice
        ...
"""


# ===globals======================
modname = "pyslice"
__version__ = "1.6.5"
LINEENDS = '\r\n'


# --option args--
debug_p = 0

global input_file
input_file = "pyslice.ini"

# ---other---

# Didn't want to have an infinite amount of jobs.
# This is just for if user makes max_threads <= 0.
total_processes = 64
numargs = 2
speciallines = {}

# ===utilities====================


def msg(txt):
    sys.stdout.write(txt)
    sys.stdout.flush()


def debug(ftn, txt):
    if debug_p:
        tmp = "".join([modname, '.', ftn, ':', txt, '\n'])
        sys.stdout.write(tmp)
        sys.stdout.flush()


def fatal(ftn, txt):
    tmp = "".join([modname, '.', ftn, ':FATAL:', txt, '\n'])
    raise SystemExit(tmp)


def usage():
    print(__doc__)


def mask(charlist):
    """
    Construct a mask suitable for s.translate,
    which marks letters in charlist as "t" and ones not as "b".
    Used by 'istext' function to identify text files.
    """
    maskvar = ""
    for i in range(256):
        if chr(i) in charlist:
            maskvar = maskvar + "t"
        else:
            maskvar = maskvar + "b"
    return maskvar


ascii7bit = "".join(
    list(map(chr, list(range(32, 127))))) + "\r\n\t\b"
ascii7bit_mask = mask(ascii7bit)


def istext(filename, check=1024, mask=ascii7bit_mask):
    """
    Returns true if the first check characters in file
    are within mask, false otherwise.
    """
    with open(filename, 'r') as filep:
        s = filep.read(check)
    s = s.translate(mask)
    if s.find("b") != -1:
        return 0
    return 1


def assignment(var1, var2):
    try:
        return var1
    except:
        return var2

# User-defined Exceptions


class NumberConfigSectionsError(Exception):
    pass


class ConfigFileNotFoundError(Exception):
    pass


class RequiredSectionNotFoundError(Exception):
    pass


class NotValidTypeError(Exception):
    pass


class TemplatePathNotFoundError(Exception):
    pass

# ====================================


class Pyslice(object):
    # ---class variables---
    # --------------------------

    def __init__(self):
        # ---instance variables---
        pass

    # --------------------------
    def read_config(self, min_sections, max_sections, req_sections_list):
        """ Reads the pyslice.ini file.

        """

        # Can we find pyslice.ini?
        pyslice_ini = os.path.join(os.getcwd(), input_file)
        if not os.access(pyslice_ini, os.F_OK | os.R_OK):
            raise ConfigFileNotFoundError(
                "%s was not found or not readable ***" % (input_file,))
        # Read it in.
        config_dict = configparser.ConfigParser()
        config_dict.read(pyslice_ini)

        # Is pyslice.ini minimally error free?
        num_sections = len(config_dict.sections())
        if num_sections < min_sections or num_sections > max_sections:
            raise NumberConfigSectionsError(
                "pyslice.ini must have between %d and %d sections." %
                (min_sections, max_sections))
        for sec in req_sections_list:
            if not config_dict.has_section(sec):
                raise RequiredSectionNotFoundError(
                    "pyslice.ini requires the [%s] section." % sec)

        # Return pyslice.ini as dictionary.
        return config_dict

    def dequote(self, in_str):
        """ Removes quotes around strings in the configuration file.
            This corrects a mistake that I would commonly make.

        """
        in_str = in_str.replace('\'', '')
        in_str = in_str.replace('\"', '')
        return in_str

    def path_correction(self, path):
        """ Corrects path separators and removes trailing path separators.

        Needed so that path separators of any OS are corrected to the
        platform running the script.

        """
        path = path.replace("/", os.sep)
        path = path.replace("\\", os.sep)
        if path[-1] == "/" or path[-1] == "\\":
            path = path[:-1]
        return path

    def create_output(self, var_set, dirname, dirs, fnames):
        var_dict = {}
        for variables in var_set[1:]:
            exec('%s = %s' % (variables[0], variables[1]))
            var_dict[variables[0]] = variables[1]

        try:
            os.makedirs(os.path.join(_output_path, _strtag))
        except OSError:
            pass
        for dirn in dirs:
            infilepath = os.path.join(dirname, dirn)
            rel_dir = infilepath.replace(_template_path + os.sep, '')
            try:
                os.makedirs(os.path.join(_output_path, _strtag, rel_dir))
            except OSError:
                pass
        for files in fnames:
            excludeflag = False
            for extensions in _exclude_list:
                if extensions in files:
                    excludeflag = True
            if excludeflag:
                continue
            infilepath = os.path.join(dirname, files)
            rel_dir = infilepath.replace(_template_path + os.sep, '')
            outfilepath = os.path.join(_output_path, _strtag, rel_dir)

            # Is this a text file?  If so, just open as a template
            if istext(infilepath):
                with open(infilepath, 'r') as inputf:
                    with open(outfilepath, 'w') as output:
                        shutil.copystat(infilepath, outfilepath)

                        filetotalizer = []
                        # Search for _keywordvarname_keyword and replace with
                        # appropriate value.
                        escaped_keyword = re.escape(_keyword)
                        keyword_search = re.compile(escaped_keyword)
                        search_for = re.compile(
                            escaped_keyword + '(.*?)' + escaped_keyword)

                        for lineraw in inputf:
                            line = lineraw
                            # Active comments first...
                            if CODE == line[:len(CODE)]:
                                filetotalizer.append(line)
                                words = line.split('|')
                                # There is the possibility that a datafile
                                # would WANT to use | So fix words up to have
                                # 1 or two items...

                                # if block length is missing... too cold
                                if len(words) == 1:
                                    linetemplate = words[0]
                                    blocklen = 1

                                # if template includes | ... too hot
                                # right now MUST include block length
                                if len(words) > numargs:
                                    linetemplate = words[:-2].join('|')
                                    blocklen = int(words[-1])

                                # ... just right
                                if len(words) == 2:
                                    linetemplate = words[0]
                                    blocklen = int(words[1])

                                linetemplate = linetemplate[len(CODE):]

                                # Process special directives
                                if '~' == linetemplate[1]:
                                    # Only want to open file once by checking
                                    # dictionary
                                    lookupno, filename = words[0].split()[1:3]
                                    lookupno = lookupno.split('~')[1]
                                    try:
                                        speciallines[filename]
                                    except KeyError:
                                        speciallines.setdefault(filename, {})
                                        with open(filename, 'r') as lout:
                                            for line in lout:
                                                # Handle comments and blank lines
                                                if '#' == line[0]:
                                                    continue
                                                (recno,
                                                 sep,
                                                 stemplate) = line.partition('|')
                                                if not stemplate:
                                                    continue
                                                stemplate = stemplate.rstrip(
                                                    LINEENDS)
                                                speciallines[filename][recno] = (
                                                    stemplate)
                                    line_sub = speciallines[filename][
                                        lookupno].format(**var_dict)

                                else:
                                    line_sub = linetemplate.format(**var_dict)

                                for blockline in range(blocklen):
                                    linein = next(inputf)
                                    nline = []
                                    for index, char in enumerate(line_sub):
                                        if char == '*':
                                            nline.append(linein[index])
                                        else:
                                            nline.append(line_sub[index])
                                    filetotalizer.append(''.join(nline))
                            elif keyword_search.search(line):
                                matchline = line
                                for matches in search_for.findall(matchline):
                                    for var_name in var_dict:
                                        if var_name in matches:
                                            # replace variable name with number
                                            match = matches.replace(
                                                var_name, str(
                                                    var_dict[var_name]))
                                            # evaluate Python statement with
                                            # eval
                                            match = eval(match)
                                            # replace variable with calculated
                                            # value
                                            matchwhat = re.escape(
                                                _keyword + matches + _keyword)
                                            matchline = re.sub(matchwhat,
                                                               str(match),
                                                               matchline,
                                                               count=1)
                                filetotalizer.append(matchline)
                            else:
                                filetotalizer.append(line)
                        output.write(''.join(filetotalizer))
            else:
                try:
                    if filecmp.cmp(infilepath, outfilepath) is False:
                        shutil.copy(infilepath, outfilepath)
                except OSError:
                    shutil.copy(infilepath, outfilepath)
                continue

    # Runs the command *com in a new thread.
    def start_thread_process(self, *com):
        com = ' '.join(com)
        com = shlex.split(com)
        # PYSLICE can be used in subprocess to do different things if script
        # is run outside of Pyslice.
        os.environ['PYSLICE'] = '1'
        if os.name != 'nt':
            p = subprocess.Popen(com,
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT,
                                 close_fds=True)
        else:
            # close_fds is not supported on Windows
            p = subprocess.Popen(com,
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT)

        chstdouterr = p.communicate()[0]

        if _keep_log is True:
            with open('pyslice.log', 'w') as fo:
                fo.write(str(chstdouterr))

    def run(self):
        global _strtag
        global _output_path
        global _keyword
        global _template_path
        global _exclude_list
        global _keep_log
        global CODE

        ftn = "Pyslice.run"
        debug(ftn, "hello, world")

        if len(sys.argv) == 1:

            toss = """
            Pyslice will replace all files in the output directories
            with the same name as files in the template directory.  If
            this is not what you want, move the files as necessary.  Pyslice
            also does not check if the permutation schedule is the same and
            may not use the same number of output directories from previous
            pyslice runs.

            'Press "Enter" to continue . . .' """
            _ = input(toss)

        # Read the configuration file and set appropriate variables.
        configuration = self.read_config(4, 100, ["paths", "flags", "program"])
        _template_path = self.dequote(configuration.get("paths",
                                                        "template_path"))
        _output_path = self.dequote(configuration.get("paths", "output_path"))
        _keyword = self.dequote(configuration.get("flags", "keyword"))
        max_threads = configuration.getint("flags", "max_threads")
        try:
            COMMENT_CODE = configuration.get('flags', 'comment')
        except:
            COMMENT_CODE = 'qwerty'
        try:
            DT_CODE = assignment(
                configuration.get('flags', 'active_comment'), '')
        except:
            DT_CODE = 'qwerty'
        CODE = COMMENT_CODE + DT_CODE
        _exclude_list = []
        if configuration.has_option('flags', 'exclude_copy'):
            _exclude_list = eval(configuration.get('flags', 'exclude_copy'))
        if max_threads <= 0:
            max_threads = total_processes
        flat_dirs = configuration.getboolean("flags", "flat_dirs")
        try:
            _keep_log = configuration.getboolean("flags", "keep_log")
        except:
            _keep_log = True

        program = self.dequote(configuration.get("program", "program"))

        # Remove the standard configuration sections to leave all of the
        # variables.
        section_list = configuration.sections()
        del section_list[section_list.index("paths")]
        del section_list[section_list.index("flags")]
        del section_list[section_list.index("program")]

        # Make sure to clean up the paths.
        _template_path = os.path.abspath(self.path_correction(_template_path))
        _output_path = os.path.abspath(self.path_correction(_output_path))

        if not os.path.exists(_template_path):
            raise TemplatePathNotFoundError(
                "The template path doesn't exists at '%s'" % (_template_path))

        # Put all variable names from configuration file into key_list.
        # Create list (from each variable) of lists (from start, stop, incr).
        list_list = []
        for variable in section_list:
            var_type = configuration.get(variable, "type")
            var_list = []

            # Monte Carlo
            if var_type == "montecarlo":
                # Cheat by using list type
                var_list.append(".%s" % (variable,))
                # Find out distribution
                distribution = 'random.' + configuration.get(variable,
                                                             "distribution")
                samples = configuration.getint(variable, "samples")
                for _ in range(samples):
                    var_list.append(eval(distribution))
            # Arithmetic
            elif var_type == "arithmetic":
                var_list.append("+%s" % (variable,))
            # Geometric
            elif var_type == "geometric":
                var_list.append("*%s" % (variable,))
            # List
            elif var_type == "list":
                var_list.append(".%s" % (variable,))
                for i in eval(configuration.get(variable, "values_list")):
                    var_list.append(i)
            else:
                raise NotValidTypeError(
                    "'%s' is not a valid type - "
                    "['arithmetic', 'geometric', 'list', "
                    "or 'montecarlo']" % (var_type,))

            # Arithmetic and Geometric types have the same variables
            if var_type == "arithmetic" or var_type == "geometric":
                start = configuration.getfloat(variable, "start")
                stop = configuration.getfloat(variable, "stop")
                increment = configuration.getfloat(variable, "increment")

                for i in [start, stop, increment]:
                    tmpi = i
                    if i == math.floor(i):
                        tmpi = int(i)
                    var_list.append(tmpi)

            var_list = [str(i) for i in var_list]
            list_list.append(var_list)

        list_list = [' '.join(i) for i in list_list]

        # This does the cartesian of all of the parameter values.
        pyspg_obj = pyspg.ParamParser(list_list)

        # nset will contain ['directory', [var, var_value], [var1, var1_value],
        # ...]
        nset = []
        # Loop reorganizes the output from PySPG and retrieves the actual
        # values.

        nmax = {}
        allints = {}
        for _ in pyspg_obj:
            tmp = []
            # Had to add the 'limit=None' in order to get directories created
            # for the last variable.  Is this a bug in PySPG?
            tmp.append(pyspg_obj.directory_tree(limit=None))
            for i in list(pyspg_obj.actual_values.items()):
                try:
                    nval = int(i[1])
                except ValueError:
                    allints[i[0]] = False
                    try:
                        nval = float(i[1])
                    except ValueError:
                        nval = i[1]
                if isinstance(nval, int):
                    if nmax.setdefault(i[0], float('-inf')) < nval:
                        nmax[i[0]] = nval
                tmp.append([i[0], nval])
            nset.append(tmp)

        while 1:
            try:
                if sys.argv[1] == 'y':
                    break
            except IndexError:
                pass

            toss = ('Configuration results in %s permutations. '
                    'Continue? (y/n) > ') % (len(nset),)
            inp = input(toss)
            if not inp:
                continue
            inp = inp[0]
            if inp == 'y' or inp == 'Y':
                break
            if inp == 'n' or inp == 'N':
                return
            continue

        nlen = len(str(len(nset)))
        for var_index, var_set in enumerate(nset):
            # Create label for output directories
            if flat_dirs:
                _strtag = str(var_index + 1).zfill(nlen)
            else:
                _strtag = os.path.curdir + os.path.sep
                for ivar in var_set[1:]:
                    if ivar[0] in allints:
                        fstr = "{0}-{1}{2}"
                    else:
                        fstr = "{0}-{1:0" + str(math.ceil(math.log10(nmax[ivar[0]] + 1))) + "d}{2}"
                    _strtag = _strtag + fstr.format(ivar[0],
                                                    ivar[1],
                                                    os.path.sep)

            # Create the files and directories from the template
            # Walks the _template_path directory structure
            # os.path.walk(_template_path, self.create_output, var_set)
            for root, dirs, files in os.walk(_template_path):
                self.create_output(var_set, root, dirs, files)

            # Wait until there are less than max_threads.
            while _threading.activeCount() > max_threads:
                time.sleep(0.1)

            abs_path = os.path.join(_output_path, _strtag)

            os.chdir(abs_path)

            a = _threading.Thread(target=self.start_thread_process,
                                  args=(program,))
            a.start()


# =============================
class Usage(Exception):

    def __init__(self, msg):
        self.msg = msg


def main(argv=None):
    if argv is None:
        argv = sys.argv

    option_dict = {
        'debug': 0,
        'file': '',
        }
    try:
        opts, _ = getopt.getopt(argv[1:],
                                'hvdf:',
                                ['help', 'version', 'debug', 'file='])
    except getopt.error as msg:
        raise Usage(msg)

    for opt in opts:
        if opt[0] == '-h' or opt[0] == '--help':
            print(modname + ": version=" + __version__)
            usage()
            sys.exit(0)
        elif opt[0] == '-v' or opt[0] == '--version':
            print(modname + ": version=" + __version__)
            sys.exit(0)
        elif opt[0] == '-d' or opt[0] == '--debug':
            option_dict['debug'] = 1
            sys.argv.remove(opt[0])
        elif opt[0] == '-f' or opt[0] == '--file':
            option_dict['file'] = opt[1]
            argv.remove(opt[0])
            argv.remove(opt[1])

    # ---make the object and run it---
    main_x = Pyslice()
    main_x.run()


if __name__ == '__main__':
    sys.exit(main())



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import

from future import standard_library
standard_library.install_aliases()
from builtins import filter
import sys
import os.path
import os

from .ParamParser import *


class TeXParser(ParamParser):
    replacing_rules = {
        "\s": "_",
              "\S": "^",
              "\f{Symbol}": " ",
              "\\f{}": " ",
              "\\N": "   "
    }

    def __agr2tex(agr_str):
        ret = agr_str
        for i in replacing_rules:
            ret = ret.replace(i, replacing_rules[i])

        return ret

    def __init__(self, comm, var_name):

        commands = [i.strip()
                    for i in comm if (i.strip()[0] != "#" and i.strip() != "")]
        # print commands

        pTmp = ParamParser(commands)
        try:
            lastvar_idx = pTmp.entities.index(var_name)
        except:
            sys.stdout = sys.stderr
            print("last_var= ", var_name)
            print("var=", lastvar_idx)
            print("exiting... -presumible error: last var you provided does not")
            print(" exist among the variables-")
            sys.exit()

        self.parser_original = ParamParser(commands[:lastvar_idx])

        lastvarpos = commands.index(
            filter(
                lambda x: x.strip()[0] in "+*/-.", commands).pop()
        )

    def __tex(self, outname="plots.tex", plotnames=[]):
        """
        for the actual values of the iterator self.parser_original, dumps a agr
        """
        cwd = os.path.abspath(os.path.curdir)
        #
        # Directory where the AGR will be located
        #
        thepath = self.parser_original.directory_tree(None)
        #
        #

        os.chdir(thepath)
        fOut = open(outname, "w")

        os.system("~/opt/bin/cts-print.sh")
        cout = sys.stdout
        sys.stdout = fOut

        print("\\documentclass[12pt]{article}")
        print()
        print("\\usepackage{graphicx}")
        print()
        print("\\begin{document}")
        print()
        print()
        print("Gr\\'aficos generados con los siguientes par\\'ametros:")
        print("\\begin{itemize}")
        actual_values = self.parser_original.actual_values
        for i in actual_values:
            print("\\item    { \\tt",)
            if actual_values[i] == '':
                posunder = i.find("_")
                if posunder > 0:
                    posblank = i.find("0")
                    print("\\begin{verbatim}")
                    print(i,)
                    print("\\end{verbatim}")
                else:
                    print(i,)
            else:
                print(" $ %s = %s $" % (i, actual_values[i]),)
            print(""" }   """)
        print("\\end{itemize}")
        print()
        print()
        ac_floats = 0
        for i in plotnames:
            print("\\begin{figure}[!ht]")
            print("\\begin{center}")
            print("\\includegraphics[height=10cm,angle=-90]{%s.eps}" % (i))
            print("\\end{center}")
            print("\\end{figure}")
            ac_floats += 1
            if ac_floats % 12 == 0:
                print("\\clearpage")

        print("\\end{document}")
        fOut.close()
        sys.stdout = cout
        os.system("latex %s" % outname)
        comandoexec = "dvips -o %s.ps  %s" % (
            os.path.splitext(outname)[0], os.path.splitext(outname)[0] + ".dvi")
        print(comandoexec)
        os.system(comandoexec)

        os.chdir(cwd)

    def doit(self, outname="plots.tex", plotnames=[]):
        """
         Suquencially dumps all the plotnames (a list of .agr files)
        """
        for i in self.parser_original:
            self.__tex(outname, plotnames)



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

#
# IMPLEMENTS A GRACE DATASET         ###########
#

from builtins import str
from builtins import map
from past.utils import old_div
from builtins import object
class GraceDataSet(object):
    basdefault = {
        "hidden": "false",
        "type": "xy",
        "baseline type": 0,
        "comment": '"Cols 1:2"',
        "legend": '" "'}

    bassymbol = {
        " ": 1,
        "size": 0.65,
        "color": 1,
        "pattern": 1,
        "fill color": 1,
        "fill pattern": 0,
        "linewidth": 1.,
        "linestyle": 1,
        "char": 65,
        "char font": 0,
        "skip": 0}

    basline = {
        "type": 1,
        "linestyle": 1,
        "linewidth": 2.,
        "color": 1,
        "pattern": 1}

    basfill = {
        "type": 0,
        "rule": 0,
        "color": 1,
        "pattern": 1}

    def __init__(self, n=0, d=[], gT="xy"):
        self.data = d
        self.name = n

        # copia todos los diccionarios
        self.default = self.basdefault.copy()
        self.symbol = self.bassymbol.copy()
        self.line = self.basline.copy()
        self.fill = self.basfill.copy()
        self.default["type"] = gT

    def __str__(self):
        return "\n".join(
            [
                "@    s" + str(self.name) + " " + a + " " + str(b)
                for a, b in list(self.default.items())
            ]
        ) + "\n" + "\n".join(
            [
                "@    s" + str(self.name) + " symbol " + a + " " + str(b)
                for a, b in list(self.symbol.items())
            ]
        ) + "\n" + "\n".join(
            [
                "@    s" + str(self.name) + " line " + a + " " + str(b)
                for a, b in list(self.line.items())
            ]
        ) + "\n" + "\n".join(
            [
                "@    s" + str(self.name) + " fill " + a + " " + str(b)
                for a, b in list(self.fill.items())
            ]
        )


#
# IMPLEMENTS A GRACE GRAPH           #########
#
class GraceGraph(object):
    bdefault = {
        " ": "on",
        "hidden": "false",
        "type": "XY",
        "stacked": "false",
        "bar hgap": "0.000000"}

    bworld = {
        "xmin": 0.,
        "xmax": 1.,
        "ymin": 0.,
        "ymax": 1.}

    bview = {
        "xmin": 0.18,
        "xmax": 1.21,
        "ymin": 0.13,
        "ymax": 0.88}

    btitle = {
        " ": '" "',
        "font": 0,
        "size": 1.5,
        "color": 1}

    bsubtitle = {
        " ": '" "',
        "font": 0,
        "size": 1.,
        "color": 1}

    baxes = {
        "scale": "Normal",
        "invert": "off"}

    baxis = {
        " ": "on",
        "type zero": "false",
        "offset": "0.0 , 0.0",
        "bar": "on",
        "bar color": 1,
        "bar linestyle": 1,
        "bar linewidth": 1.,
        "label": '"x"',
        "label layout": "para",
        "label place": "auto",
        "label char size": 1.9,
        "label font": 1,
        "label color": 1,
        "label place": "normal",
        "tick": "on",
        "tick major": 0.5,
        "tick minor ticks": 1,
        "tick default": 6,
        "tick place rounded": "true",
        "tick": "in",
        "tick major size": 1.,
        "tick major color": 1,
        "tick major linewidth": 1.0,
        "tick major linestyle": 1,
        "tick major grid": "off",
        "tick minor color": 1,
        "tick minor linewidth": 1.0,
        "tick minor linestyle": 1,
        "tick minor grid": "off",
        "tick minor size": 0.5,
        "ticklabel": "on",
        "ticklabel format": "general",
        "ticklabel prec": 5,
        "ticklabel formula": '""',
        "ticklabel append": '""',
        "ticklabel prepend": '""',
        "ticklabel angle": 0,
        "ticklabel skip": 0,
        "ticklabel stagger": 0,
        "ticklabel place": "normal",
        "ticklabel offset": "auto",
        "ticklabel offset": "0.00 , 0.01",
        "ticklabel start type": "auto",
        "ticklabel start": 0.,
        "ticklabel stop type": "auto",
        "ticklabel stop": 0.,
        "ticklabel char size": 1.36,
        "ticklabel font": 0,
        "ticklabel color": 1,
        "tick place": "both",
        "tick spec type": "none"}

    blegend = {
        "": "on",
        "loctype": "view",
        " ": "0.85, 0.8",
        "box color": 1,
        "box pattern": 1,
        "box linewidth": 1.,
        "box linestyle": 1,
        "box fill color": 0,
        "box fill pattern": 1,
        "font": 0,
        "char size": 1.,
        "color": 1,
        "length": 4,
        "vgap": 1,
        "hgap": 1,
        "invert": "false"}

    bframe = {
        "type": 0,
        "linestyle": 1,
        "linewidth": 1.0,
        "color": 1,
        "pattern": 1,
        "background color": 0,
        "background pattern": 0}

    def __init__(self, n=0):
        self.default = self.bdefault.copy()
        self.world = self.bworld.copy()
        self.view = self.bview.copy()
        self.title = self.btitle.copy()
        self.subtitle = self.bsubtitle.copy()
        self.xaxes = self.baxes.copy()
        self.yaxes = self.baxes.copy()

        self.xaxis = self.baxis.copy()
        self.yaxis = self.baxis.copy()
        self.legend = self.blegend.copy()
        self.frame = self.bframe.copy()

        self.name = n

    def __str__(self):
        return "\n".join(
            [
                "@g" + str(self.name) + " " + a + " " + str(b)
                for a, b in list(self.default.items())
            ]) + "\n@with g" + str(self.name) + "\n" + "\n".join(
            [
                "@    world " + a + " " + str(b)
                for a, b in list(self.world.items())
            ]) + "\n" + "\n".join(
            [
                "@    view " + a + " " + str(b)
                for a, b in list(self.view.items())
            ]) + "\n" + "\n".join(
            [
                "@    title " + a + " " + str(b)
                for a, b in list(self.title.items())
            ]) + "\n" + "\n".join(
            [
                "@    subtitle " + a + " " + str(b)
                for a, b in list(self.subtitle.items())
            ]) + "\n" + "\n".join(
            [
                "@    xaxes " + a + " " + str(b)
                for a, b in list(self.xaxes.items())
            ]) + "\n" + "\n".join(
            [
                "@    yaxes " + a + " " + str(b)
                for a, b in list(self.yaxes.items())
            ]) + "\n" + "\n".join(
            [
                "@    xaxis " + a + " " + str(b)
                for a, b in list(self.xaxis.items())
            ]
        ) + "\n" + "\n".join(
            [
                "@    yaxis " + a + " " + str(b)
                for a, b in list(self.yaxis.items())
            ]) + "\n" + "\n".join(
            [
                "@    legend " + a + " " + str(b)
                for a, b in list(self.legend.items())
            ]) + "\n" + "\n".join(
            [
                "@    frame " + a + " " + str(b)
                for a, b in list(self.frame.items())
            ]
        ) + "\n"


#
# IMPLEMENTS A GRACE DOC        ################
#
class GraceDocument(object):
  #
  #:::~ Constants
    basdefault = {"version": 50100}

    baspage = {
        'size': "792,612",
        'scroll': "5%",
        'inout': "5%"}

    basmap_font = {
        0: ('"Times-Roman"', '"Times-Roman"'),
        1: ('"Times-Italic"', '"Times-Italic"'),
        2: ('"Times-Bold"', '"Times-Bold"'),
        3: ('"Times-BoldItalic"', '"Times-BoldItalic"'),
        4: ('"Helvetica"', '"Helvetica"'),
        5: ('"Helvetica-Oblique"', '"Helvetica-Oblique"'),
        6: ('"Helvetica-Bold"', '"Helvetica-Bold"'),
        7: ('"Helvetica-BoldOblique"', '"Helvetica-BoldOblique"'),
        8: ('"Courier"', '"Courier"'),
        9: ('"Courier-Oblique"', '"Courier-Oblique"'),
        10: ('"Courier-Bold"', '"Courier-Bold"'),
        11: ('"Courier-BoldOblique"', '"Courier-BoldOblique"'),
        12: ('"Symbol"', '"Symbol"'),
        13: ('"ZapfDingbats"', '"ZapfDingbats"')}

    basmap_color = {
        0: ((255, 255, 255), '"white"'),
        1: ((0, 0, 0), '"black"'),
        2: ((255, 0, 0), '"red"'),
        3: ((0, 255, 0), '"green"'),
        4: ((0, 0, 255), '"blue"'),
        5: ((255, 255, 0), '"yellow"'),
        6: ((188, 143, 143), '"brown"'),
        7: ((220, 220, 220), '"grey"'),
        8: ((148, 0, 211), '"violet"'),
        9: ((0, 255, 255), '"cyan"'),
        10: ((255, 0, 255), '"magenta"'),
        11: ((255, 165, 0), '"orange"'),
        12: ((114, 33, 188), '"indigo"'),
        13: ((103, 7, 72), '"maroon"'),
        14: ((64, 224, 208), '"turquoise"'),
        15: ((0, 139, 0), '"green4"')}

    def __init__(self):
        # copies all the dicts
        self.datasets = {}
        self.graph = GraceGraph()

        self.default = self.basdefault.copy()
        self.page = self.baspage.copy()
        self.map_font = self.basmap_font.copy()
        self.map_color = self.basmap_color.copy()

    def __str__(self):
        return "\n".join(
            [
                "@" + a + " " + str(b)
                for a, b in list(self.default.items())
            ]
        ) + "\n" + "\n".join(
            [
                "@page " + a + " " + str(b)
                for a, b in list(self.page.items())
            ]
        ) + "\n" + "\n".join(
            [
                "@map font " + str(a) + " to " + b + ", " + c
                for a, (b, c) in list(self.map_font.items())
            ]
        ) + "\n" + "\n".join(
            [
                "@map color " +
                str(a) +
                " to (" + str(b) + "," + str(
                    c) + "," + str(d) + "), " + str(e)
                for a, ((b, c, d), e) in list(self.map_color.items())
            ]
        )

    #
    #
    #
    #
    #
    #

    def set_data(self, ls, legend="", graphType="xy"):
        newds = GraceDataSet(len(self.datasets), ls, graphType)
        newds.symbol[" "] = len(self.datasets) + 1
        newds.line["color"] = len(self.datasets) + 1
        newds.symbol["color"] = len(self.datasets) + 1
        if legend == "":
            legend = "y" + str(len(self.datasets))
        legend = '"' + legend + '"'
        newds.default["legend"] = legend
        self.datasets[len(self.datasets)] = newds

    def set_world(self, minx, maxx, miny, maxy, tickx=None, ticky=None):

        if minx == maxx:
            minx -= .5
            maxx += .5

        if miny == maxy:
            miny -= .5
            maxy += .5
        if not tickx:
            tickx = old_div((maxx - minx), 4.)
        if not ticky:
            ticky = old_div((maxy - miny), 4.)

        self.graph.world["xmin"] = minx
        self.graph.world["xmax"] = maxx
        self.graph.world["ymin"] = miny
        self.graph.world["ymax"] = maxy

        self.graph.xaxis["tick major"] = tickx
        self.graph.yaxis["tick major"] = ticky

    def getRoundedValues(self, scale, a2, a1):
        import math
        sign1 = sign2 = 1
        if a1 < 0:
            sign1 = -1
        if a2 < 0:
            sign2 = -1
        aa1 = abs(a1)
        aa2 = abs(a2)
        scale1 = math.floor(math.log10(max(1e-10, aa1)))
        scale2 = math.floor(math.log10(max(1e-10, aa2)))

        int1 = eval(("%e" % aa1)[0]) + 1
        int2 = eval(("%e" % aa2)[0]) - 1

        if scale is "Normal":
            propose1 = sign1 * int1 * 10 ** scale1
            propose2 = sign2 * int2 * 10 ** scale2
            skip = abs(old_div((int1 * 10 ** scale1 - int2 * 10 ** scale2), 4))
            return propose2, propose1, old_div(propose1, 4)
        else:
            propose1 = 10 ** (scale1 + 1)
            propose2 = 10 ** (scale2 - 1)

            return propose2, propose1, 10 ** int((scale1 - scale2) + 1)

    def autoscale(self, autoscaleaxis="xy"):
        minx = 1e10
        maxx = -1e10
        miny = 1e10
        maxy = -1e10

        for it in list(self.datasets.values()):
            colx = [a[0] for a in it.data]
            coly = [a[1] for a in it.data]
            if len(colx) > 0:
                minx = min(minx, min(colx))
                maxx = max(maxx, max(colx))
            if len(coly) > 0:
                miny = min(miny, min(coly))
                maxy = max(maxy, max(coly))

        minx, maxx, tickx = self.getRoundedValues(
            self.graph.xaxes["scale"], minx, maxx)
        miny, maxy, ticky = self.getRoundedValues(
            self.graph.yaxes["scale"], miny, maxy)
        if autoscaleaxis.find('x') >= 0:
            self.graph.world["xmin"] = minx
            self.graph.world["xmax"] = maxx
            self.graph.xaxis["tick major"] = tickx

        if autoscaleaxis.find('y') >= 0:
            self.graph.world["ymin"] = miny
            self.graph.world["ymax"] = maxy

            self.graph.yaxis["tick major"] = ticky

    def set_labels(self, stx, sty):
        self.graph.xaxis["label"] = '"' + stx + '"'
        self.graph.yaxis["label"] = '"' + sty + '"'

    def set_title(self, st):
        self.graph.subtitle[" "] = '"' + st + '"'

    def dump(self, outStream=None):
        import sys
        old_stdout = sys.stdout
        if outStream:
            sys.stdout = outStream

        print(self)
        print(self.graph)
        for i in list(self.datasets.keys()):
            print(self.datasets[i])
        for i in list(self.datasets.keys()):
            print("@target G" + str(self.graph.name) + ".S" + str(i))
            print("@type " + str(self.datasets[i].default["type"]))
            print("\n".join(["\t".join(map(str, a)) for a in self.datasets[i].data]))
            print("\n&")

        sys.stdout = old_stdout

    def set_scale(self, x="Normal", y="Normal"):
        self.graph.xaxes["scale"] = x
        self.graph.yaxes["scale"] = y

#  Example of use
#
#  g1=GraceDocument()
#
#  g1.set_data([[0.12,0.001],[0.1,0.5],[0.2,0.6],[0.3,0.8]],"foo")
#  g1.set_data([[.1,.1],[0.1,0.4],[0.2,0.5],[0.3,0.9]])

#  g1.autoscale()
#  g1.set_labels("x","y")
#  g1.set_scale(x="Logarithmic",y="Logarithmic")
# g1.set_world(-1,1,-1,1)
#  g1.set_title("Foo")
#  g1.dump()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#===configuration=============
import os, sys, re, getopt, shutil
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()
history = open('CHANGES.rst').read().replace('.. :changelog:', '')

pkgname = 'pyslice'
modname = 'setup.py'
version = open("VERSION").readline().strip()
exec_prefix = sys.exec_prefix

install_requires = [
        'future',
        'configparser',
        ]
#=============================

#----------------------
setup(  # ---meta-data---
    name="pyslice",
    version=version,
    description="Pyslice is a templating engine to easily create data sets for parametric modeling.",
    long_description=readme + '\n\n' + history,

    author="Tim Cera, P.E.",
    author_email="tim@cerazone.net",
    url="http://timcera.bitbucket.io/pyslice/docsrc/index.html",
    install_requires=install_requires,
    packages=['pyslice',
        'pyslice.pyslice_lib',
        'pyslice.pyslice_lib.PySPG'],
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts':
            ['pyslice=pyslice:main']
        },
    classifiers=[
          # Get strings from
          # http://pypi.python.org/pypi?%3Aaction=list_classifiers
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Science/Research',
          'Intended Audience :: End Users/Desktop',
          'Environment :: Console',
          'License :: OSI Approved :: BSD License',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 3',
          'Topic :: Scientific/Engineering :: Information Analysis',
          'Topic :: Scientific/Engineering',
          'Topic :: Text Processing :: General',
      ],
    )
#==============================



from __future__ import absolute_import
from __future__ import division
#! /usr/bin/python

from builtins import map
from builtins import str
from builtins import range
from past.utils import old_div
from . import PyGrace


def loadData(s):
    try:
        fIn = open(s)
    except:
        import sys
        sys.stderr.write("error!!! opening file: %s...\n" % s)
        sys.stderr.write("gently quitting...\n")
        sys.exit()
    try:
        return [
            list(map(float, strLine.replace("D", "E").strip().split()))
            for strLine in fIn.readlines() if strLine.find("nan") == -1 and strLine.strip() != "" and strLine.find("inf") == -1
        ]
    except:
        import sys
        sys.stderr.write("error!!! in file: %s...\n" % s)
        sys.stderr.write("not correctly formatted, perhaps...\n")
        sys.stderr.write("gracefully quitting...\n")
        sys.exit()


def dumpData(s, data):
    try:
        fout = open(s, "w")
    except:
        import sys
        sys.stderr.write("error!!! opening for write file: %s...\n" % s)
        sys.stderr.write("gently quitting...\n")
        sys.exit()

    for line in data:
        fout.write("\t".join([str(i) for i in line]) + "\n")


def loadXY(s, xCol=0, yCol=1):
    data = loadData(s)
    out = []
    for line in data:
        try:
            out.append([line[xCol], line[yCol]])
        except:
            import sys
            sys.stderr.write(
                "skipping line %d, wrong size\n" % (data.index(line)))
    return out
#    return [
#           [line[xCol],line[yCol] ]
#           for line in data
#         ]
#  except:
#    import sys
#    sys.stderr.write("error!!! in file: %s...\n"%s)
#    sys.stderr.write("not enough columns, or inconsistent datafile. asked for columns %d and %d, where there are only %d..\n"%(xCol,yCol,len(data[0])))
#    sys.stderr.write("not enough columns, or inconsistent datafile. asked for columns %d and %d, where there are only %d..\n"%(xCol,yCol,len(data[-1])))
#    sys.stderr.write("gracefully quitting...\n")
#    sys.exit()

# try: #
#    return [
#           [line[xCol],line[yCol] ]
#           for line in data
#         ]
#  except:
#    import sys
#    sys.stderr.write("error!!! in file: %s...\n"%s)
#    sys.stderr.write("not enough columns, or inconsistent datafile. asked for columns %d and %d, where there are only %d..\n"%(xCol,yCol,len(data[0])))
#    sys.stderr.write("not enough columns, or inconsistent datafile. asked for columns %d and %d, where there are only %d..\n"%(xCol,yCol,len(data[-1])))
#    sys.stderr.write("gracefully quitting...\n")
#    sys.exit()


def loadXYZ(s, xCol=0, yCol=1, zCol=2):
    return [
        [line[xCol], line[yCol], line[zCol]]
        for line in loadData(s)
    ]


def loadY(s, yCol=1):
    fIn = open(s)
    import sys

    try:
        lines = fIn.readlines()
#    sys.stderr.write("%d   ,"%len(lines) )
        ls = [
            float(line.strip().split()[yCol])
            for line in lines if len(line) > 1
        ]
#    print len(ls)
        return ls
    except:
        import os.path
        sys.stderr.write(" ... error reading file: %s, %d\n" %
                         (os.path.realpath(s), os.path.isfile(s)))
#    sys.stderr.write("exiting...\n")
#    sys.exit(2)
        return [0]


def transformXY(ls, xFormula="x", yFormula="y"):
    import math
    xf = xFormula.replace("x", "(x)")
    xf = xf.replace("y", "(y)")
    yf = yFormula.replace("x", "(x)")
    yf = yf.replace("y", "(y)")

    lsOut = []

    for line in ls:
        try:
            lsOut.append(
                [
                    eval(
                        xf.replace(
                            "x", str(line[0])).replace("y", str(line[1]))),
                    eval(
                        yf.replace("y", str(line[1])).replace("x", str(line[0])))
                ]
            )

        except:
            import sys
            sys.stderr.write("skipping line: " + str(line) + "\n")
    return lsOut


def transformXYZ(ls, xFormula="x", yFormula="y", zFormula="z"):
    xf = xFormula.replace("x", "(x)")
    yf = yFormula.replace("y", "(y)")
    zf = zFormula.replace("z", "(z)")

    return [
        [
            eval(xf.replace("x", str(line[0]))),
            eval(yf.replace("y", str(line[1]))),
            eval(zf.replace("z", str(line[2])))
        ]
        for line in ls
    ]

try:
    import numpy
except:
    import sys
    sys.stderr.write("numpy Package Not found...\n")
    sys.stderr.write("Not histogram capability present...\n")
else:
    def histogram(data, nbins, maxboxsize=10000, minx=-1e64, maxx=1e64):
    #
    # pythonize this code!!!!!!
    #
        if len(data) == 0:
            data = [0]
        import math
        mina = max(min(data), minx)
        maxa = min(max(data), maxx)
#  mina=min(data)
#  maxa=max(data)
        boxsize = min(maxboxsize, old_div((maxa - mina), nbins))
        out = [0] * int(math.ceil(old_div((maxa - mina), boxsize)))
        import sys
        for idata in data:
            try:
                pos = int(math.floor(old_div((idata - mina), boxsize)))
                if pos == len(out):
                    pos = len(out) - 1
                sys.stderr.write("%d\n" % pos)
                out[pos] += 1
            except:

                sys.stderr.write(
                    "histogram error!!! len(data) = %d \n" % len(out))
                sys.stderr.write("\n".join([
                                           "%s = %s" % (
                                               str(k), str(locals()[k]))
                                           for k in list(locals().keys()) if k is not "data"
                                           ]) + "\n"
                                 )

                sys.exit()
        suma = Numeric.sum(out)

        out3 = [
            old_div(x, suma) for x in out
        ]
        out2 = []

        for i in range(nbins):
            out2.append(
                [float(i) / float(nbins) * (maxa - mina) + mina, out3[i]])
        return out2



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import

#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2003
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit www.gnu.org
#

from future import standard_library
standard_library.install_aliases()
from builtins import filter
from builtins import object
from . import PyGrace
from . import Load
from .ParamParser import *
import sys
import os.path


class MultiAgrizer(object):

    def __init__(self, comm, var_name):
        """
        Systematically plots the data generated by comm considering var_name as the last one
        the following ones are generated in the same plot
        """
        self.xcol = 0
        self.ycol = 1
        self.zcol = 2
        self.xformula = "x"
        self.yformula = "y"
        self.zformula = "z"
        self.xlabel = "x"
        self.ylabel = "y"
        self.xscale = "Normal"
        self.yscale = "Normal"
        self.plottype = "xy"
        self.autoscale_vars = "xy"
        self.xmin = 0
        self.xmax = 1
        self.xticks = 0.2
        self.ymin = 0
        self.ymax = 1
        self.yticks = 0.2
        self.title = ""
        self.dictLabels = None

        commands = [i.strip()
                    for i in comm if (i.strip()[0] != "#" and i.strip() != "")]
        # print commands

        pTmp = ParamParser(commands)
        try:
            lastvar_idx = pTmp.entities.index(var_name)
        except:
            sys.stdout = sys.stderr
            print("last_var= ", var_name)
            print("var=", lastvar_idx)
            print("exiting... -presumible error: last var you provided does not")
            print(" exist among the variables-")
            sys.exit()

        self.parser_original = ParamParser(commands[:lastvar_idx])

        lastvarpos = commands.index(
            filter(
                lambda x: x.strip()[0] in "+*/-.", commands).pop()
        )

        self.parser_last = ParamParser(commands[lastvar_idx:lastvarpos])

    def set_world(self, xmin, xmax, ymin, ymax, xticks=None, yticks=None):
        """
          sets the bounds for the
        """
        self.xmin = xmin
        self.xmax = xmax
        self.ymin = ymin
        self.ymax = ymax
        self.xticks = xticks
        self.yticks = yticks

    def __agr(self, outname, inname):
        """
        for the actual values of the iterator self.parser_original, dumps a agr
        """
        cwd = os.path.abspath(os.path.curdir)
        #
        # Directory where the AGR will be located
        #
        thepath = self.parser_original.directory_tree(None)
        #
        #

        os.chdir(thepath)
        fOut = open(outname, "w")

        g1 = PyGrace.GraceDocument()
        #
        # Genera los paths donde estan los files
        #
        self.parser_last.reset()
        for i_state in self.parser_last:
            fname = self.parser_last.directory_tree(None) + inname
            legend = " ".join([
                "%s=%s" % (key, i_state[key])
                for key in i_state
            ])
#       print i_state

            if self.plottype == "xy":
                ls = Load.loadXY(fname, self.xcol, self.ycol)
                ls2 = Load.transformXY(ls, self.xformula, self.yformula)
            if self.plottype in ["xydy", "xydx"]:
                ls = Load.loadXYZ(fname, self.xcol, self.ycol, self.zcol)
                ls2 = Load.transformXYZ(
                    ls, self.xformula, self.yformula, self.zformula)
            g1.set_data(ls2, legend, self.plottype)

#
        g1.set_labels(self.xlabel, self.ylabel)
        g1.set_scale(self.xscale, self.yscale)
        g1.set_title(self.title)
        g1.autoscale()
        g1.dump(fOut)
        #
        #
        #

        os.chdir(cwd)

    def doit(self, outname="out.agr", inname="out.dat"):
        """
        given the commands, for all the possible values, do the plotting according the given rules
        """

        for i in self.parser_original:
    #    print self.parser_original
            self.__agr(outname, inname)


#
#



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2003
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit www.gnu.org
#
#

from builtins import str
from builtins import range
from builtins import object
from past.utils import old_div
import sys
from math import *

from .Load import *
from .ParamParser import *


class MeanCalculation(object):

    """
      When a simulation is run with a given number of repetitions what is needed is a
      tool for calculating the mean values. This is that tool
    """

    def __init__(self, lsLines):
        ls2 = [
            i for i in lsLines
            if i.strip()[0] != "#"
        ]

        self.pp_varying = ParamParser(ls2)

    def searchinfile(self, fname, st):
        return [
            i.split()
            for i in open(fname, "r").readlines()
            if i.split()[0] == st

        ]

    def mean(self, fin_name="out.dat", fout_name="out.mean"):

        dirname = self.pp_varying.directory_tree()

        fsearch = "%s%s" % (dirname, fin_name)
        fout = "%s%s" % (dirname, fout_name)

        try:
            all_data = loadData(fsearch)
        except:
            print("Error! '%s' file doesn't exist or not enough permissions..." % fsearch)
            sys.exit()

        try:
            columnas = list(range(len(all_data[0])))
        except:

            sys.stderr.write("mean-> error! in directory '%s'\n" % dirname)
            sys.stderr.write("all_data = %s\n" % str(all_data))

        tmpDict = {}

        for i in all_data:
            tmpDict[i[0]] = 0
        valuesX = list(tmpDict.keys())
        valuesX.sort()

        xDict = {}
        nPoints = {}

        import copy
        zeros = [0 for i in columnas]

        for i in valuesX:
            xDict[i] = copy.copy(zeros)
            nPoints[i] = 0.

        for row in all_data:
            if len(row) == len(all_data[0]):
                nPoints[row[0]] += 1.
                vec = xDict[row[0]]
                for j in columnas:
                    vec[j] += row[j]

        dataOut = []

        for x in valuesX:
            if nPoints[x] != 0:
                dataOut.append(
                    [
                        old_div(xDict[x][j], nPoints[x])
                        for j in columnas
                    ]
                )

        try:
            dumpData(fout, dataOut)
        except:
            print("Error! Couldn't create output file '%s' ..." % fsearch)
            sys.exit()

    def doit(self, fin_name="out.dat", fout_name="mean.dat"):
        for i in self.pp_varying:
            self.mean(fin_name, fout_name)


#
#



#! /usr/bin/python

from __future__ import print_function
from __future__ import absolute_import

#
#
# :::~ Author: Claudio Juan Tessone <tessonec@imedea.uib.es> (c) 2002-2003
#
# Distributed According to GNU Generic Purpose License (GPL)
# Please visit www.gnu.org
#

from future import standard_library
standard_library.install_aliases()
from builtins import filter
from builtins import str
from builtins import object
import sys
import os.path

from . import PyGrace
from . import Load
from .ParamParser import *


class MatrixPlotter(object):

    save_mtv = False

    def __init__(self, comm):
        self.xcol = 1
        self.xformula = "x"

        self.yformula = "y"

        self.zmin = None
        self.zmax = None
        self.nz = 50

        commands = [i.strip()
                    for i in comm if (i.strip()[0] != "#" and i.strip() != "")]
        try:
            pTmp = ParamParser(commands)
            var_name = pTmp.variables_list[-2].get_varname()
            print(var_name)
            lastvar_idx = pTmp.entities.index(var_name)
        except:
            sys.stdout = sys.stderr
            print("last_var= ", var_name)
            print("var=", lastvar_idx)
            print("exiting... -presumible error: last var you provided does not")
            print(" exist among the variables-")
            sys.exit()

        self.parser_original = ParamParser(commands[:lastvar_idx])

        lastvarpos = commands.index(
            filter(
                lambda x: x.strip()[0] in "+*/-.", commands).pop()
        )

        self.parser_last = ParamParser(commands[lastvar_idx:lastvarpos])

        self.iter2 = pTmp.variables_list[-2]
        self.iter1 = pTmp.variables_list[-1]

    def __matrix(self, outname, inname):
        """
        for the actual values of the iterator self.parser_original, dumps a agr
        """
        cwd = os.path.abspath(os.path.curdir)
        #
        # Directory where the AGR will be located
        #
        thepath = self.parser_original.directory_tree(None)
        #
        #

        os.chdir(thepath)
        #
        # Genera los paths donde estan los files
        #
        self.parser_last.reset()
        outputStr = ""
        for i_state in self.parser_last:
            fname = self.parser_last.directory_tree(None) + inname
            ls = Load.loadXY(fname, self.xcol, self.ycol)
            ls2 = [str(i[1])
                   for i in Load.transformXY(ls, self.xformula, self.yformula)]
            outputStr += "\t".join(ls2) + "\n"
        #
        #
        #

        fOut = open(outname, "w")
        if self.save_mtv:
            mtvStr = "$ DATA=CONTOUR\n"
            mtvStr += "%" + " NX=%d    XMIN=%f   XMAX=%f XLABEL=%s \n" % (
                len(self.iter1.data), self.iter1.data[0], self.iter1.data[-1], self.iter1.get_varname())
            mtvStr += "%" + " NY=%d    YMIN=%f   YMAX=%f YLABEL=%s \n" % (
                len(self.iter2.data), self.iter2.data[0], self.iter2.data[-1], self.iter2.get_varname())
            mtvStr += "%" + " NSTEPS=%d    \n" % (self.nz)
            if self.zmin != None:
                mtvStr += "%" + " ZMIN=%f    \n" % (self.cmin)
            if self.zmax != None:
                mtvStr += "%" + " ZMAX=%f    \n" % (self.cmax)
            mtvStr += "%" + "contstyle=4\n"
            mtvStr += "%" + "interpolate=2\n"
            mtvStr += "%" + "contfill=True\n"

            fOut.write(mtvStr)

        fOut.write(outputStr)
        if self.save_mtv:
            mtvStr = "$ END"

            fOut.write(mtvStr)
        fOut.close()
#     print outputStr
        os.chdir(cwd)

    def doit(self, outname="out.mtrx", inname="mean.dat", ycol=1):
        """
        given the commands, for all the possible values, do the plotting according the given rules
        """
        self.ycol = ycol
        for i in self.parser_original:
    #    print self.parser_original
            self.__matrix(outname, inname)


#
#
